@@ -55,6 +55,8 @@
 
 #define N_GCONF_PREFS 7
 
+#define NEVER_SENSITIVE "never_sensitive"
+
 static const char* KEY_HOUR_FORMAT   = "hour_format";
 static const char* KEY_SHOW_SECONDS  = "show_seconds";
 static const char* KEY_SHOW_DATE     = "show_date";
@@ -803,13 +805,16 @@ config_date (BonoboUIComponent *uic,
 
 	screen = gtk_widget_get_screen (cd->applet);
 
-	/* FIXME add GST, etc. */
+	/* FIXME add other time config tools. */
 	if (cd->config_tool && try_config_tool (screen, cd->config_tool))
 		return;
 
 	else if (try_config_tool (screen, "redhat-config-date"))
 		return;
 		
+	else if (try_config_tool (screen, "time-admin"))
+		return;
+		
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
@@ -1119,11 +1124,37 @@ fill_clock_applet (PanelApplet *applet)
 	return TRUE;
 }
 
+static void
+setup_writability_sensitivity (ClockData *clock, GtkWidget *w, GtkWidget *label, const char *key)
+{
+	char *fullkey;
+	GConfClient *client;
+
+	client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key
+		(PANEL_APPLET (clock->applet), key);
+
+	if ( ! gconf_client_key_is_writable (client, fullkey, NULL)) {
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+		if (label != NULL) {
+			g_object_set_data (G_OBJECT (label), NEVER_SENSITIVE,
+					   GINT_TO_POINTER (1));
+			gtk_widget_set_sensitive (label, FALSE);
+		}
+	}
+
+	g_free (fullkey);
+}
+
 static void
 set_data_sensitive_cb (GtkWidget *w,
 		      GtkWidget *wid)
 {
-	gtk_widget_set_sensitive (wid, TRUE);
+	if ( ! g_object_get_data (G_OBJECT (wid), NEVER_SENSITIVE))
+		gtk_widget_set_sensitive (wid, TRUE);
 }
 
 static void
@@ -1453,6 +1484,14 @@ display_properties_dialog (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT (cd->props), "response",
 			  G_CALLBACK (properties_response_cb), cd);
 
+	/* Now set up the sensitivity based on gconf key writability */
+	setup_writability_sensitivity (cd, option_menu, label, KEY_HOUR_FORMAT);
+	setup_writability_sensitivity (cd, option_menu, label, KEY_UNIX_TIME);
+	setup_writability_sensitivity (cd, option_menu, label, KEY_INTERNET_TIME);
+	setup_writability_sensitivity (cd, showseconds, NULL, KEY_SHOW_SECONDS);
+	setup_writability_sensitivity (cd, showdate, NULL, KEY_SHOW_DATE);
+	setup_writability_sensitivity (cd, use_gmt_time, NULL, KEY_GMT_TIME);
+
 	/* sets up atk relation  */
 	list = g_slist_append (NULL, twelvehour);
 	list = g_slist_append (list, twentyfourhour);
