@@ -127,16 +127,8 @@ panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
-	switch (info->type) {
-	case PANEL_OBJECT_LAUNCHER:
+	if (info->type == PANEL_OBJECT_LAUNCHER)
 		panel_launcher_delete (info->data);
-		break;
-	case PANEL_OBJECT_DRAWER:
-		panel_drawer_delete (info->data);
-		break;
-	default:
-		break;
-	}
 
 	if (info->widget) {
 		GtkWidget *widget = info->widget;
@@ -179,9 +171,6 @@ applet_callback_callback (GtkWidget      *widget,
 			launcher_properties (
 				menu->info->data, screen);
 
-		else if (!strcmp (menu->name, "help"))
-			panel_show_help (screen, "wgospanel.xml", "gospanel-16");
-
 		else if (!strcmp (menu->name, "help_on_app"))
 			launcher_show_help (menu->info->data, screen);
 		break;
@@ -465,6 +454,14 @@ panel_applet_create_menu (AppletInfo  *info,
 	if (!commie_mode) {
 		GtkWidget *image;
 		gboolean   locked;
+		gboolean   lockable;
+		gboolean   movable;
+		gboolean   removable;
+
+
+		lockable = panel_applet_lockable (info);
+		movable = panel_applet_can_freely_move (info);
+		removable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
 
 		locked = panel_widget_get_applet_locked (panel_widget, info->widget);
 
@@ -479,6 +476,7 @@ panel_applet_create_menu (AppletInfo  *info,
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, removable);
 
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
@@ -486,14 +484,15 @@ panel_applet_create_menu (AppletInfo  *info,
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (panel_applet_lock), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, lockable);
 		
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
 		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"));
-		gtk_widget_set_sensitive (menuitem, !locked);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, !locked && movable);
 
 		g_assert (info->move_item == NULL);
 
@@ -669,7 +668,8 @@ typedef struct {
 	guint            locked : 1;
 } PanelAppletToLoad;
 
-static GSList *panel_applets_to_load = NULL;
+static GSList  *panel_applets_to_load = NULL;
+static gboolean panel_applet_have_load_idle = FALSE;
 
 static void
 free_applet_to_load (PanelAppletToLoad *applet)
@@ -683,6 +683,18 @@ free_applet_to_load (PanelAppletToLoad *applet)
 	g_free (applet);
 }
 
+gboolean
+panel_applet_on_load_queue (const char *id)
+{
+	GSList *li;
+	for (li = panel_applets_to_load; li != NULL; li = li->next) {
+		PanelAppletToLoad *applet = li->data;
+		if (strcmp (applet->id, id) == 0)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static gboolean
 panel_applet_load_idle_handler (gpointer dummy)
 {
@@ -691,8 +703,10 @@ panel_applet_load_idle_handler (gpointer dummy)
 	PanelWidget       *panel_widget;
 	GSList            *l;
 
-	if (!panel_applets_to_load)
+	if (!panel_applets_to_load) {
+		panel_applet_have_load_idle = FALSE;
 		return FALSE;
+	}
 
 	for (l = panel_applets_to_load; l; l = l->next) {
 		applet = l->data;
@@ -708,6 +722,7 @@ panel_applet_load_idle_handler (gpointer dummy)
 			free_applet_to_load (l->data);
 		g_slist_free (panel_applets_to_load);
 		panel_applets_to_load = NULL;
+		panel_applet_have_load_idle = FALSE;
 		return FALSE;
 	}
 
@@ -828,7 +843,10 @@ panel_applet_load_queued_applets (void)
 	panel_applets_to_load = g_slist_sort (panel_applets_to_load,
 					      (GCompareFunc) panel_applet_compare);
 
-	g_idle_add (panel_applet_load_idle_handler, NULL);
+	if ( ! panel_applet_have_load_idle) {
+		g_idle_add (panel_applet_load_idle_handler, NULL);
+		panel_applet_have_load_idle = TRUE;
+	}
 }
 
 static G_CONST_RETURN char *
@@ -876,7 +894,9 @@ panel_applet_save_position (AppletInfo *applet_info,
 	const char        *profile;
 	const char        *key;
 	const char        *toplevel_id;
+	char              *old_toplevel_id;
 	gboolean           right_stick;
+	gboolean           locked;
 	int                position;
 
 	g_return_if_fail (applet_info != NULL);
@@ -905,25 +925,33 @@ panel_applet_save_position (AppletInfo *applet_info,
 	
 	panel_widget = PANEL_WIDGET (applet_info->widget->parent);
 
-	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
-	gconf_client_set_string (client, key, toplevel_id, NULL);
+	/* FIXME: Instead of getting keys, comparing and setting, there
+	   should be a dirty flag */
 
-	right_stick = panel_is_applet_right_stick (applet_info->widget);
+	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
+	old_toplevel_id = gconf_client_get_string (client, key, NULL);
+	if (old_toplevel_id == NULL || strcmp (old_toplevel_id, toplevel_id) != 0)
+		gconf_client_set_string (client, key, toplevel_id, NULL);
+	g_free (old_toplevel_id);
+
+	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (
 			key_type, profile, id, "panel_right_stick");
-	gconf_client_set_bool (client, key, right_stick, NULL);
+	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != right_stick)
+		gconf_client_set_bool (client, key, right_stick, NULL);
 
 	position = panel_applet_get_position (applet_info);
 	if (right_stick && !panel_widget->packed)
 		position = panel_widget->size - position;
 
 	key = panel_gconf_full_key (key_type, profile, id, "position");
-	gconf_client_set_int (client, key, position, NULL);
+	if (gconf_client_get_int (client, key, NULL) != position)
+		gconf_client_set_int (client, key, position, NULL);
 	
+	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (key_type, profile, id, "locked");
-	gconf_client_set_bool (client, key, 
-			       panel_widget_get_applet_locked (panel_widget, applet_info->widget),
-			       NULL);
+	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != locked)
+		gconf_client_set_bool (client, key, locked, NULL);
 }
 
 const char *
@@ -1089,3 +1117,36 @@ panel_applet_get_position (AppletInfo *applet)
 
 	return applet_data->pos;
 }
+
+/* True if all the keys relevant to moving are writable
+   (position, toplevel_id, panel_right_stick) */
+gboolean
+panel_applet_can_freely_move (AppletInfo *applet)
+{
+	GConfClient *client = panel_gconf_get_client ();
+	const char *profile = panel_profile_get_name ();
+	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	const char *key;
+       
+	key = panel_gconf_full_key (key_type, profile, applet->id, "position");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_gconf_full_key (key_type, profile, applet->id, "toplevel_id");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_gconf_full_key (key_type, profile, applet->id, "panel_right_stick");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	return TRUE;
+}
+
+/* True if the locked flag is writable */
+gboolean
+panel_applet_lockable (AppletInfo *applet)
+{
+	GConfClient *client = panel_gconf_get_client ();
+	const char *profile = panel_profile_get_name ();
+	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	const char *key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
+	return gconf_client_key_is_writable (client, key, NULL);
+}
