@@ -206,7 +206,7 @@ destroy_drawer (GtkWidget *widget,
 	drawer->properties = NULL;
 
 	if (drawer->toplevel)
-		panel_toplevel_detach (drawer->toplevel);
+		gtk_widget_destroy (GTK_WIDGET (drawer->toplevel));
 	drawer->toplevel = NULL;
 
 	if (drawer->close_timeout_id)
@@ -486,11 +486,11 @@ create_drawer_applet (PanelToplevel    *toplevel,
 
 	if (!pixmap || !pixmap [0]) {
 		drawer->pixmap = NULL;
-		drawer->button = button_widget_new_from_stock (PANEL_STOCK_DRAWER, -1,
+		drawer->button = button_widget_new_from_stock (PANEL_STOCK_DRAWER,
 							       TRUE, orientation);
 	} else {
 		drawer->pixmap = g_strdup (pixmap);
-		drawer->button = button_widget_new (drawer->pixmap, -1,
+		drawer->button = button_widget_new (drawer->pixmap,
 						    TRUE, orientation);
 	}
 
@@ -602,6 +602,7 @@ load_drawer_applet (char          *toplevel_id,
 	PanelOrientation  orientation;
 	PanelToplevel    *toplevel = NULL;
 	Drawer           *drawer = NULL;
+	PanelWidget      *panel_widget;
 
 	orientation = panel_toplevel_get_orientation (parent_toplevel);
 
@@ -617,9 +618,11 @@ load_drawer_applet (char          *toplevel_id,
 	if (!drawer)
 		return;
 
+	panel_widget = panel_toplevel_get_panel_widget (parent_toplevel);
+
 	drawer->info = panel_applet_register (drawer->button, drawer,
 					      (GDestroyNotify) free_drawer,
-					      panel_toplevel_get_panel_widget (parent_toplevel),
+					      panel_widget,
 					      locked, pos, exactpos,
 					      PANEL_OBJECT_DRAWER, id);
 
@@ -632,6 +635,8 @@ load_drawer_applet (char          *toplevel_id,
 				G_CALLBACK (drawer_button_size_allocated), drawer);
 
 	panel_widget_add_forbidden (panel_toplevel_get_panel_widget (drawer->toplevel));
+	panel_widget_set_applet_expandable (panel_widget, GTK_WIDGET (drawer->button), FALSE, TRUE);
+	panel_widget_set_applet_size_constrained (panel_widget, GTK_WIDGET (drawer->button), TRUE);
 
 	gtk_tooltips_set_tip (panel_tooltips, drawer->button, drawer->tooltip, NULL);
 	gtk_window_present (GTK_WINDOW (drawer->toplevel));
@@ -680,12 +685,6 @@ panel_drawer_create (PanelToplevel *toplevel,
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
 }
 
-void
-panel_drawer_delete (Drawer *drawer)
-{
-	panel_profile_delete_toplevel (drawer->toplevel);
-}
-
 void
 drawer_load_from_gconf (PanelWidget *panel_widget,
 			gboolean     locked,
