@@ -5,11 +5,6 @@
 
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomeui/gnome-ditem-edit.h>
 
 #include "menu-ditem.h"
@@ -26,7 +21,9 @@ ditem_properties_clicked (GtkWidget *w, int response, gpointer data)
 	GnomeDesktopItem *ditem = data;
 
 	if (response == GTK_RESPONSE_HELP) {
-		panel_show_help ("launchers", NULL);
+		panel_show_help (
+			gtk_window_get_screen (GTK_WINDOW (w)),
+			"wgospanel.xml", "gospanel-16");
 	} else if (response == REVERT_BUTTON) {
 		if (ditem != NULL)
 			gnome_ditem_edit_set_ditem (dee, ditem);
@@ -83,8 +80,8 @@ ditem_properties_changed (GtkWidget *dedit, gpointer data)
 	if (timeout != 0)
 		g_source_remove (timeout);
 
-	/* Will delay save for after 5 seconds */
-	timeout = g_timeout_add (5 * 1000,
+	/* Will delay save for after 2 seconds */
+	timeout = g_timeout_add (2 * 1000,
 				 ditem_properties_apply_timeout,
 				 dedit);
 
@@ -94,9 +91,9 @@ ditem_properties_changed (GtkWidget *dedit, gpointer data)
 
 
 static void
-ditem_properties_close (GtkWidget *widget, gpointer data)
+ditem_properties_close (GtkWidget *dialog,
+			GtkWidget *dedit)
 {
-	GtkWidget *dedit = data;
 	const char *saving_error;
 	gpointer timeout_data = g_object_get_data (G_OBJECT (dedit),
 						   "apply_timeout");
@@ -114,10 +111,18 @@ ditem_properties_close (GtkWidget *widget, gpointer data)
 
 	saving_error = g_object_get_data (G_OBJECT (dedit), "SavingError");
 
-	if (saving_error != NULL) {
-		panel_error_dialog ("cannot_save_entry",
-				    _("<b>Cannot save changes to launcher</b>\n\n"
-				      "Details: %s"), saving_error);
+	if (saving_error) {
+		char *msg;
+		msg = g_strdup_printf ("<b>%s</b>\n\n%s: %s",
+			_("Cannot save changes to launcher"),
+			_("Details"), saving_error);
+
+		panel_error_dialog (
+			gtk_window_get_screen (GTK_WINDOW (dialog)),
+			"cannot_save_entry",
+			msg);
+
+		g_free (msg);
 	}
 }
 
@@ -126,7 +131,7 @@ is_item_writable (const char *loc, const char *dir)
 {
 	if (loc != NULL) {
 		/* if old style kde link file, don't allow editing */
-		if (is_ext (loc, ".kdelnk"))
+		if (g_str_has_suffix (loc, ".kdelnk"))
 			return FALSE;
 		if (panel_is_uri_writable (loc))
 			return TRUE;
@@ -162,31 +167,30 @@ set_ditem_sensitive (GtkDialog *dialog,
 
 GtkWidget *
 panel_edit_dentry (const char *loc,
-		   const char *dir)
+		   const char *dir,
+		   GdkScreen  *screen)
 {
-	GtkWidget *dialog;
-	GtkWidget *dedit;
 	GnomeDesktopItem *ditem;
+	GtkWidget        *dialog;
+	GtkWidget        *dedit;
 	
 	g_return_val_if_fail (loc != NULL, NULL);
 
-	ditem = gnome_desktop_item_new_from_uri (loc,
-						 0 /* flags */,
-						 NULL /* error */);		 
+	ditem = gnome_desktop_item_new_from_uri (loc, 0, NULL);		 
 
-	/* watch the enum at the top of the file */
-	dialog = gtk_dialog_new_with_buttons (_("Launcher Properties"),
-					      NULL /* parent */,
-					      0 /* flags */,
-					      GTK_STOCK_HELP,
-					      GTK_RESPONSE_HELP,
-					      GTK_STOCK_REVERT_TO_SAVED,
-					      REVERT_BUTTON,
-					      GTK_STOCK_CLOSE,
-					      GTK_RESPONSE_CLOSE,
-					      NULL);
+	dialog = gtk_dialog_new_with_buttons (
+				_("Launcher Properties"),
+				NULL, 0 /* flags */,
+				GTK_STOCK_HELP,
+				GTK_RESPONSE_HELP,
+				GTK_STOCK_REVERT_TO_SAVED,
+				REVERT_BUTTON,
+				GTK_STOCK_CLOSE,
+				GTK_RESPONSE_CLOSE,
+				NULL);
 
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+	gtk_dialog_set_default_response (
+			GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
 	dedit = gnome_ditem_edit_new ();
 
@@ -194,8 +198,9 @@ panel_edit_dentry (const char *loc,
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    dedit, TRUE, TRUE, 0);
 
-	gtk_window_set_wmclass(GTK_WINDOW(dialog),
-			       "desktop_entry_properties","Panel");
+	gtk_window_set_wmclass (GTK_WINDOW (dialog),
+			        "desktop_entry_properties","Panel");
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	
 	g_object_set_data_full (G_OBJECT (dedit), "location",
 				g_strdup (loc),
@@ -208,13 +213,11 @@ panel_edit_dentry (const char *loc,
 			     GNOME_DITEM_EDIT (dedit),
 			     loc, dir);
 
-	g_signal_connect (G_OBJECT (dedit), "changed",
-			    G_CALLBACK (ditem_properties_changed),
-			    NULL);
+	g_signal_connect (dedit, "changed",
+			  G_CALLBACK (ditem_properties_changed), NULL);
 
-	g_signal_connect (G_OBJECT (dialog), "destroy",
-			    G_CALLBACK (ditem_properties_close),
-			    dedit);
+	g_signal_connect (dialog, "destroy",
+			  G_CALLBACK (ditem_properties_close), dedit);
 
 	g_object_set_data (G_OBJECT (dialog), "GnomeDItemEdit", dedit);
 
@@ -239,7 +242,9 @@ panel_edit_dentry (const char *loc,
 }
 
 GtkWidget *
-panel_edit_direntry (const char *dir, const char *dir_name)
+panel_edit_direntry (const char *dir,
+		     const char *dir_name,
+		     GdkScreen  *screen)
 {
 	GtkWidget *dialog;
 	GtkWidget *dedit;
@@ -248,24 +253,22 @@ panel_edit_direntry (const char *dir, const char *dir_name)
 	
 	dirfile = g_strconcat (dir, "/", ".directory", NULL);
 
-	ditem = gnome_desktop_item_new_from_uri (dirfile,
-						 0 /* flags */,
-						 NULL /* error */);
+	ditem = gnome_desktop_item_new_from_uri (dirfile, 0, NULL);
 
-	/* watch the enum at the top of the file */
-	dialog = gtk_dialog_new_with_buttons (_("Launcher Properties"),
-					      NULL /* parent */,
-					      0 /* flags */,
-					      GTK_STOCK_HELP,
-					      GTK_RESPONSE_HELP,
-					      GTK_STOCK_REVERT_TO_SAVED,
-					      REVERT_BUTTON,
-					      GTK_STOCK_CLOSE,
-					      GTK_RESPONSE_CLOSE,
-					      NULL);
+	dialog = gtk_dialog_new_with_buttons (
+				_("Launcher Properties"),
+				NULL, 0 /* flags */,
+				GTK_STOCK_HELP,
+				GTK_RESPONSE_HELP,
+				GTK_STOCK_REVERT_TO_SAVED,
+				REVERT_BUTTON,
+				GTK_STOCK_CLOSE,
+				GTK_RESPONSE_CLOSE,
+				NULL);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"desktop_entry_properties", "Panel");
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	
 	dedit = gnome_ditem_edit_new ();
 	gtk_widget_show (dedit);
@@ -310,13 +313,11 @@ panel_edit_direntry (const char *dir, const char *dir_name)
 			     GNOME_DITEM_EDIT (dedit),
 			     dirfile, NULL);
 
-	g_signal_connect (G_OBJECT (dedit), "changed",
-			    G_CALLBACK (ditem_properties_changed),
-			    NULL);
+	g_signal_connect (dedit, "changed",
+			  G_CALLBACK (ditem_properties_changed), NULL);
 
-	g_signal_connect (G_OBJECT (dialog), "destroy",
-			    G_CALLBACK (ditem_properties_close),
-			    dedit);
+	g_signal_connect (dialog, "destroy",
+			  G_CALLBACK (ditem_properties_close), dedit);
 
 	g_object_set_data (G_OBJECT (dialog), "GnomeDItemEdit", dedit);
 
@@ -378,15 +379,17 @@ get_unique_name (const char *dir, const char *name)
 	int i;
 	char *schema;
 	char *full, *test;
+	char *nameext = g_strdup_printf ("%s.desktop", name);
 
 	schema = get_schema (dir);
 
-	full = g_strdup_printf ("%s/%s.desktop", dir, name);
+	full = g_build_path ("/", dir, nameext, NULL);
 	if ( ! panel_uri_exists (full)) {
-		test = g_strdup_printf ("%s:%s.desktop", schema, name);
+		test = g_strdup_printf ("all-%s:%s", schema, nameext);
 		if ( ! panel_uri_exists (test)) {
 			g_free (schema);
 			g_free (test);
+			g_free (nameext);
 			return full;
 		}
 		g_free (test);
@@ -395,13 +398,20 @@ get_unique_name (const char *dir, const char *name)
 
 	i = 2;
 	for (;;) {
-		full = g_strdup_printf ("%s/%s%d.desktop", dir, name, i++);
+		g_free (nameext);
+		nameext = g_strdup_printf ("%s%d.desktop", name, i++);
 
+		/* randomize further same name desktops */
+		if (i > 5)
+			i = rand ();
+
+		full = g_build_path ("/", dir, nameext, NULL);
 		if ( ! panel_uri_exists (full)) {
-			test = g_strdup_printf ("%s:%s.desktop", schema, name);
+			test = g_strdup_printf ("all-%s:%s", schema, nameext);
 			if ( ! panel_uri_exists (test)) {
 				g_free (schema);
 				g_free (test);
+				g_free (nameext);
 				return full;
 			}
 			g_free (test);
@@ -433,10 +443,10 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 
 	/* check for valid name */
 	if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
-		dialog = panel_error_dialog_with_parent
-			(GTK_WINDOW (d),
-			 "cannot_create_launcher",
-			 _("You have to specify a name for the launcher."));
+		dialog = panel_error_dialog (
+				gtk_window_get_screen (GTK_WINDOW (d)),
+				"cannot_create_launcher",
+				_("You have to specify a name for the launcher."));
 		g_signal_connect_swapped (G_OBJECT (dialog),
 					  "destroy",
 					  G_CALLBACK (panel_pop_window_busy),
@@ -449,11 +459,10 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC))) ||
 	    (gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_LINK &&
 	     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
-		gnome_desktop_item_unref (ditem);
-		dialog = panel_error_dialog_with_parent
-			(GTK_WINDOW (d),
-			 "cannot_create_launcher",
-			 _("You have to specify a valid URL or command."));
+		dialog = panel_error_dialog (
+				gtk_window_get_screen (GTK_WINDOW (d)),
+				"cannot_create_launcher",
+				_("You have to specify a valid URL or command."));
 		g_signal_connect_swapped (G_OBJECT (dialog),
 					  "destroy",
 					  G_CALLBACK (panel_pop_window_busy),
@@ -462,10 +471,15 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	}
 
 	/* assume we are making a new file */
-	name = g_strdup (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME));
+	name = g_filename_from_utf8 (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME),
+				     -1, NULL, NULL, NULL);
+	if (name == NULL)
+		name = g_strdup ("foo");
 
 	validate_for_filename (name);
 
+	ditem = gnome_desktop_item_copy (ditem);
+
 	loc = get_unique_name (dir, name);
 	gnome_desktop_item_set_location (ditem, loc);
 	g_free (name);
@@ -475,11 +489,18 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 				 NULL /* under */,
 				 TRUE /* force */,
 				 &error);
-	if (error != NULL) {
-		panel_error_dialog ("cannot_save_menu_item" /* class */,
-				    _("<b>Cannot save menu item to disk</b>\n\n"
-				      "Details: %s"),
-				    error->message);
+	if (error) {
+		char *msg;
+		msg = g_strdup_printf ("<b>%s</b>\n\n%s: %s",
+			_("Cannot save menu item to disk"),
+			_("Details"), error->message);
+
+		panel_error_dialog (
+			gtk_window_get_screen (GTK_WINDOW (d)),
+			"cannot_save_menu_item" /* class */,
+			msg);
+
+		g_free (msg);
 		g_clear_error (&error);
 	}
 
@@ -492,30 +513,32 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 }
 
 GtkWidget *
-panel_new_launcher (const char *item_loc)
+panel_new_launcher (const char *item_loc,
+		    GdkScreen  *screen)
 {
 	GtkWidget *dialog;
 	GtkWidget *dee;
 
 	if (!is_item_writable (item_loc, NULL)) {
-		
-		dialog = panel_error_dialog ("cannot_create_launcher",
-				             _("You can not create a new launcher at this location since the location is not writable."));
+		dialog = panel_error_dialog (
+				screen,
+				"cannot_create_launcher",
+				_("You can not create a new launcher at this location "
+				  "since the location is not writable."));
 
 		return dialog;
 	}
 
-	dialog = gtk_dialog_new_with_buttons (_("Create Launcher"),
-					      NULL /* parent */,
-					      0 /* flags */,
-					      GTK_STOCK_CANCEL,
-					      GTK_RESPONSE_CANCEL,
-					      GTK_STOCK_OK,
-					      GTK_RESPONSE_OK,
-					      NULL);
+	dialog = gtk_dialog_new_with_buttons (
+				_("Create Launcher"),
+				NULL, 0 /* flags */,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OK, GTK_RESPONSE_OK,
+				NULL);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 			       "create_menu_item", "Panel");
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	
 	dee = gnome_ditem_edit_new ();
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), dee,
@@ -541,4 +564,3 @@ panel_new_launcher (const char *item_loc)
 
 	return dialog;
 }
-
