@@ -32,20 +32,22 @@
 
 #include "applet.h"
 #include "egg-screen-exec.h"
-#include "gnome-run.h"
 #include "menu.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
 #include "panel-profile.h"
 #include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
+#include "panel-force-quit.h"
 #include "panel-util.h"
 #include "panel-session.h"
 #include "panel-globals.h"
+#include "panel-run-dialog.h"
 
 enum {
 	PROP_0,
 	PROP_ACTION_TYPE,
+	PROP_DND_ENABLED
 };
 
 struct _PanelActionButtonPrivate {
@@ -53,6 +55,8 @@ struct _PanelActionButtonPrivate {
 	AppletInfo            *info;
 
 	guint                  gconf_notify;
+	
+	guint                  dnd_enabled : 1;
 };
 
 static GObjectClass *parent_class;
@@ -64,6 +68,7 @@ static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_RUN,          "run"          },
 	{ PANEL_ACTION_SEARCH,       "search"       },
 	{ PANEL_ACTION_SCREENSHOT,   "screenshot"   },
+	{ PANEL_ACTION_FORCE_QUIT,   "force-quit"   },
 };
 
 /* Lock Screen
@@ -133,7 +138,7 @@ panel_action_logout (GtkWidget *widget)
 void
 panel_action_run_program (GtkWidget *widget)
 {
-	show_run_dialog (gtk_widget_get_screen (widget));
+	panel_run_dialog_present (gtk_widget_get_screen (widget));
 }
 
 /* Search For Files
@@ -168,6 +173,18 @@ panel_action_screenshot (GtkWidget *widget)
 				    _("Cannot execute gnome-panel-screenshot"));
 }
 
+/* Force Quit
+ */
+static void
+panel_action_force_quit (GtkWidget *widget)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+
+	panel_force_quit (screen);
+}
+
 typedef struct {
 	PanelActionButtonType   type;
 	char                   *stock_icon;
@@ -209,6 +226,11 @@ static PanelAction actions [] = {
 		PANEL_ACTION_SCREENSHOT, PANEL_STOCK_SCREENSHOT,
 		N_("Screenshot"), NULL,
 		panel_action_screenshot, NULL, NULL
+	},
+	{
+		PANEL_ACTION_FORCE_QUIT, PANEL_STOCK_FORCE_QUIT,
+		N_("Force Quit"), NULL,
+		panel_action_force_quit, NULL, NULL
 	}
 };
 
@@ -246,6 +268,9 @@ panel_action_button_get_property (GObject    *object,
 	case PROP_ACTION_TYPE:
 		g_value_set_enum (value, button->priv->type);
 		break;
+	case PROP_DND_ENABLED:
+		g_value_set_boolean (value, button->priv->dnd_enabled);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -269,6 +294,10 @@ panel_action_button_set_property (GObject      *object,
 		panel_action_button_set_type (button,
 					      g_value_get_enum (value));
 		break;
+	case PROP_DND_ENABLED:
+		panel_action_button_set_dnd_enabled (button,
+						     g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -352,29 +381,28 @@ panel_action_button_class_init (PanelActionButtonClass *klass,
 					   PANEL_TYPE_ACTION_BUTTON_TYPE,
 					   PANEL_ORIENTATION_TOP,
 					   G_PARAM_READWRITE));
+
+	g_object_class_install_property (
+			gobject_class,
+			PROP_DND_ENABLED,
+			g_param_spec_boolean ("dnd-enabled",
+					      _("Drag N' Drop enabled"),
+					      _("Whether or not drag and drop is enabled on the widget"),
+					      TRUE,
+					      G_PARAM_READWRITE));
 }
 
 static void
 panel_action_button_instance_init (PanelActionButton      *button,
 				   PanelActionButtonClass *klass)
 {
-	static GtkTargetEntry dnd_targets [] = {
-		{ "application/x-panel-applet-internal", 0, 0 }
-	};
-
 	button->priv = g_new0 (PanelActionButtonPrivate, 1);
 
 	button->priv->type = PANEL_ACTION_NONE;
 	button->priv->info = NULL;
 
 	button->priv->gconf_notify = 0;
-
-	GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
-	gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
-			     dnd_targets, 1,
-			     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
-
+	button->priv->dnd_enabled  = FALSE;
 }
 
 GType
@@ -468,6 +496,7 @@ panel_action_button_style_set (PanelActionButton *button)
 static void
 panel_action_button_load (PanelActionButtonType  type,
 			  PanelWidget           *panel,
+			  gboolean               locked,
 			  int                    position,
 			  gboolean               exactpos,
 			  const char            *id,
@@ -488,9 +517,10 @@ panel_action_button_load (PanelActionButtonType  type,
 			object_type = PANEL_OBJECT_LOGOUT;
 	}
 
-	button->priv->info = panel_applet_register (
-				GTK_WIDGET (button), NULL, NULL, panel,
-				position, exactpos, object_type, id);
+	button->priv->info = panel_applet_register (GTK_WIDGET (button),
+						    NULL, NULL,
+						    panel, locked, position,
+						    exactpos, object_type, id);
 	if (!button->priv->info) {
 		gtk_widget_destroy (GTK_WIDGET (button));
 		return;
@@ -499,6 +529,9 @@ panel_action_button_load (PanelActionButtonType  type,
 	panel_applet_add_callback (
 		button->priv->info, "help", GTK_STOCK_HELP, _("_Help"));
 
+	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
+	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
+
 	if (actions [button->priv->type].setup_menu)
 		actions [button->priv->type].setup_menu (button);
 
@@ -521,7 +554,7 @@ panel_action_button_create (PanelToplevel         *toplevel,
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_ACTION, toplevel, position);
+	id = panel_profile_prepare_object (PANEL_OBJECT_ACTION, toplevel, position, FALSE);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "action_type");
 	gconf_client_set_string (client,
@@ -541,6 +574,7 @@ panel_action_button_create (PanelToplevel         *toplevel,
 void
 panel_action_button_load_compatible (PanelObjectType  object_type,
 				     PanelWidget     *panel,
+				     gboolean         locked,
 				     int              position,
 				     gboolean         exactpos,
 				     const char      *id)
@@ -551,11 +585,12 @@ panel_action_button_load_compatible (PanelObjectType  object_type,
 
 	action_type = object_type == PANEL_OBJECT_LOGOUT ? PANEL_ACTION_LOGOUT : PANEL_ACTION_LOCK;
 
-	panel_action_button_load (action_type, panel, position, exactpos, id, TRUE);
+	panel_action_button_load (action_type, panel, locked, position, exactpos, id, TRUE);
 }
 
 void
 panel_action_button_load_from_gconf (PanelWidget *panel,
+				     gboolean     locked,
 				     int          position,
 				     gboolean     exactpos,
 				     const char  *id)
@@ -578,7 +613,7 @@ panel_action_button_load_from_gconf (PanelWidget *panel,
 
 	g_free (action_type);
 
-	panel_action_button_load (type, panel, position, exactpos, id, FALSE);
+	panel_action_button_load (type, panel, locked, position, exactpos, id, FALSE);
 }
 
 void
@@ -648,3 +683,37 @@ panel_action_button_load_from_drag (PanelToplevel *toplevel,
 
 	return retval;
 }
+
+void
+panel_action_button_set_dnd_enabled (PanelActionButton *button,
+				     gboolean           enabled)
+{
+	g_return_if_fail (PANEL_IS_ACTION_BUTTON (button));
+
+	if (!button->priv->type)
+		return; /* wait until we know what type it is */
+
+	enabled = enabled != FALSE;
+
+	if (button->priv->dnd_enabled == enabled)
+		return;
+
+	if (enabled) {
+		static GtkTargetEntry dnd_targets [] = {
+			{ "application/x-panel-applet-internal", 0, 0 }
+		};
+
+		GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
+		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
+				     dnd_targets, 1,
+				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+		gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
+						actions [button->priv->type].stock_icon);
+		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
+	} else
+		gtk_drag_source_unset (GTK_WIDGET (button));
+
+	button->priv->dnd_enabled = enabled;
+
+	g_object_notify (G_OBJECT (button), "dnd-enabled");
+}
