@@ -267,12 +267,12 @@ panel_applet_frame_load (const gchar *iid,
 				      panel, locked, position,
 				      exactpos, PANEL_OBJECT_BONOBO, id);
 
+	panel_widget_set_applet_size_constrained (panel, widget, TRUE);
+
 	frame = PANEL_APPLET_FRAME (widget);
 
 	panel_applet_frame_set_info (frame, info);
-
 	panel_applet_frame_sync_menu_state (frame);
-
 	panel_applet_frame_init_properties (frame);
 }
 
@@ -488,33 +488,6 @@ panel_applet_frame_expose (GtkWidget      *widget,
 	return FALSE;
 }
 
-static void
-panel_applet_frame_constrain_size (PanelAppletFrame *frame,
-				   GtkRequisition   *requisition)
-{
-	PanelWidget *panel;
-
-	g_return_if_fail (PANEL_IS_WIDGET (frame->priv->panel));
-
-	panel = frame->priv->panel;
-
-	switch (frame->priv->orientation) {
-	case PANEL_ORIENTATION_TOP:
-	case PANEL_ORIENTATION_BOTTOM:
-		if (requisition->height > panel->sz)
-			requisition->height = panel->sz;
-		break;
-	case PANEL_ORIENTATION_LEFT:
-	case PANEL_ORIENTATION_RIGHT:
-		if (requisition->width > panel->sz)
-			requisition->width = panel->sz;
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-}
-
 static void
 panel_applet_frame_size_request (GtkWidget      *widget,
 				 GtkRequisition *requisition)
@@ -528,7 +501,6 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 
 	if (!frame->priv->has_handle) {
 		GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
-		panel_applet_frame_constrain_size (frame, requisition);
 		return;
 	}
   
@@ -558,8 +530,6 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 		g_assert_not_reached ();
 		break;
 	}
-
-	panel_applet_frame_constrain_size (frame, requisition);
 }
 
 static void
@@ -901,8 +871,13 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 
 	gtk_widget_destroy (dialog);
 
-	if (response == GTK_RESPONSE_OK)
-		panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
+	if (response == GTK_RESPONSE_OK) {
+		/* if we can't write to applets list we can't really delete
+		   it, so we'll just ignore this.  There's nothing we can
+		   do about it. */
+		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+			panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
+	}
 }
 
 static void
