@@ -57,6 +57,7 @@ typedef struct {
 } ToplevelLocationChange;
 
 typedef const char *(*PanelProfileGetIdFunc)   (gpointer           object);
+typedef gboolean    (*PanelProfileOnLoadQueue) (const char        *id);
 typedef void        (*PanelProfileLoadFunc)    (GConfClient       *client,
 						const char        *profile_dir, 
 						PanelGConfKeyType  type,
@@ -277,6 +278,8 @@ panel_profile_find_new_id (PanelGConfKeyType type)
 			}
 	}
 
+	g_assert (retval != NULL);
+
 	for (l = existing_ids; l; l = l->next)
 		g_free (l->data);
 	g_slist_free (existing_ids);
@@ -1679,12 +1682,13 @@ panel_profile_id_exists (GSList     *id_list,
 }
 
 static void
-panel_profile_load_added_ids (GConfClient           *client,
-			      PanelGConfKeyType      type,
-			      GSList                *list,
-			      GSList                *id_list,
-			      PanelProfileGetIdFunc  get_id_func,
-			      PanelProfileLoadFunc   load_handler)
+panel_profile_load_added_ids (GConfClient            *client,
+			      PanelGConfKeyType       type,
+			      GSList                 *list,
+			      GSList                 *id_list,
+			      PanelProfileGetIdFunc   get_id_func,
+			      PanelProfileLoadFunc    load_handler,
+			      PanelProfileOnLoadQueue on_load_queue)
 {
 	GSList *added_ids = NULL;
 	GSList *l;
@@ -1693,7 +1697,8 @@ panel_profile_load_added_ids (GConfClient           *client,
 	for (l = id_list; l; l = l->next) {
 		const char *id = gconf_value_get_string (l->data);
 
-		if (!panel_profile_object_exists (list, id, get_id_func))
+		if (!panel_profile_object_exists (list, id, get_id_func) &&
+		    (on_load_queue == NULL || !on_load_queue (id)))
 			added_ids = g_slist_prepend (added_ids, g_strdup (id));
 	}
 
@@ -1771,7 +1776,8 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 				      existing_toplevels,
 				      toplevel_ids,
 				      (PanelProfileGetIdFunc) panel_profile_get_toplevel_id,
-				      (PanelProfileLoadFunc) panel_profile_load_and_show_toplevel);
+				      (PanelProfileLoadFunc) panel_profile_load_and_show_toplevel,
+				      (PanelProfileOnLoadQueue) NULL);
 
 	panel_profile_delete_removed_ids (client,
 					  PANEL_GCONF_TOPLEVELS,
@@ -1819,7 +1825,8 @@ panel_profile_object_id_list_notify (GConfClient *client,
 				      sublist,
 				      object_ids,
 				      (PanelProfileGetIdFunc) panel_applet_get_id,
-				      (PanelProfileLoadFunc) panel_profile_load_object);
+				      (PanelProfileLoadFunc) panel_profile_load_object,
+				      (PanelProfileOnLoadQueue) panel_applet_on_load_queue);
 
 	panel_profile_delete_removed_ids (client,
 					  type,
@@ -1978,7 +1985,7 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 
 		schema_key = g_strdup_printf (PANEL_SCHEMAS_DIR "/%s", 
 			                      schemas_str);
-		panel_gconf_associate_schemas_in_dir (client, dest_dir, key);
+		panel_gconf_associate_schemas_in_dir (client, dest_dir, schema_key);
 		g_free (schema_key);
 
 		new_ids = g_slist_prepend (new_ids, new_id);
@@ -2055,6 +2062,26 @@ panel_profile_copy_default_objects_for_screen (GConfClient       *client,
 	panel_profile_append_new_ids (client, type, new_objects);
 }
 
+/* FIXME:
+ *   We might want to do something more sophisticated like hardcode
+ *   the default panel setup as the fallback panels.
+ */
+static GSList *
+panel_profile_create_fallback_toplevel_list (GConfClient *client,
+					     const char  *profile_dir)
+{
+	char *id;
+	char *dir;
+
+	id = panel_profile_find_new_id (PANEL_GCONF_TOPLEVELS);
+
+	dir = g_strdup_printf ("%s/toplevels/%s", profile_dir, id);
+	panel_gconf_associate_schemas_in_dir (client, dir, PANEL_SCHEMAS_DIR "/toplevels");
+	g_free (dir);
+
+	return g_slist_prepend (NULL, id);
+}
+
 static void
 panel_profile_load_defaults_on_screen (GConfClient *client,
 				       const char  *profile_dir,
@@ -2067,6 +2094,11 @@ panel_profile_load_defaults_on_screen (GConfClient *client,
 
 	new_toplevels = panel_profile_copy_defaults_for_screen (
 				client, profile_dir, screen_n, PANEL_GCONF_TOPLEVELS);
+	if (!new_toplevels) {
+		g_warning ("Failed to load default panel configuration. panel-default-setup.entries "
+			   "hasn't been installed using gconftool-2 --load ?\n");
+		new_toplevels = panel_profile_create_fallback_toplevel_list (client, profile_dir);
+	}
 
 	for (l = new_toplevels; l; l = l->next) {
 		char       *toplevel_id = l->data;
@@ -2117,9 +2149,6 @@ panel_profile_ensure_toplevel_per_screen (GConfClient *client,
 
 	for (l = empty_screens; l; l = l->next)
 		panel_profile_load_defaults_on_screen (client, profile_dir, l->data);
-		/* FIXME: if we fail to load the defaults for whatever reason we
-		 *        should load a fallback panel.
-		 */
 
 	g_slist_free (empty_screens);
 }
